//socket 연결
const clientIo = io.connect("https://dev.knowledgetalk.co.kr:7100/SignalServer",{});

const roomIdInput = document.getElementById("roomIdInput");
const videoBox = document.getElementById("videoBox");
const printBox = document.getElementById("printBox")

const CreateRoomBtn = document.getElementById("CreateRoomBtn");
const RoomJoinBtn = document.getElementById("RoomJoinBtn");
const SDPBtn = document.getElementById("SDPBtn");

const blurBtn = document.getElementById('blur-btn');
const unblurBtn = document.getElementById('unblur-btn');
const speakerDetectBtn = document.getElementById('speaker-detect-btn');
const speakerDetectCancelBtn = document.getElementById('speaker-detect-cancel-btn');
const audioSafeBtn = document.getElementById('audio-safe-btn');
const audioSafeCancelBtn = document.getElementById('audio-safe-cancel-btn');

const CPCODE = "KP-CCC-demouser-01"
const AUTHKEY = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJuYW1lIjoidGVzdHNlcnZpY2UiLCJtYXhVc2VyIjoiMTAwIiwic3RhcnREYXRlIjoiMjAyMC0wOC0yMCIsImVuZERhdGUiOiIyMDIwLTEyLTMwIiwiYXV0aENvZGUiOiJLUC1DQ0MtdGVzdHNlcnZpY2UtMDEiLCJjb21wYW55Q29kZSI6IkxJQy0wMyIsImlhdCI6MTU5Nzk3NjQ3Mn0.xh_JgK67rNPufN2WoBa_37LzenuX_P7IEvvx5IbFZI4"

let members;
let roomId;
let userId;

let peers = {};
let streams = {};

let membersOfVideoOn = [];
let membersOfVideoOff = [];
/********************** 기타 method **********************/

//print log on page
const socketLog = (type, contents) => {
    let jsonContents = JSON.stringify(contents);
    const textLine = document.createElement("p");
    const textContents = document.createTextNode(`[${type}] ${jsonContents}`);
    textLine.appendChild(textContents);
    printBox.appendChild(textLine);
}

//send message to signaling server
const sendData = data => {
    data.cpCode = CPCODE
    data.authKey = AUTHKEY
    socketLog('send', data);
    clientIo.emit("knowledgetalk", data);
}

const deletePeers = async () => {
    for(let key in streams) {
        if (streams[key] && streams[key].getTracks()) {
            streams[key].getTracks().forEach(track => {
                track.stop();
            })

            document.getElementById(key).srcObject = null;
            document.getElementById(key).remove();
        }
    }

    for(let key in peers) {
        if (peers[key]) {
            peers[key].close();
            peers[key] = null;
        }
    }
}

//영상 출력 화면 Box 생성
const createVideoBox = id => {
    membersOfVideoOn.push(id);

    let videoContainner = document.createElement("div");
    videoContainner.classList = "multi-video";
    videoContainner.id = id;

    let videoLabel = document.createElement("p");
    let videoLabelText = document.createTextNode(id);
    videoLabel.appendChild(videoLabelText);

    videoContainner.appendChild(videoLabel);

    let multiVideo = document.createElement("video");
    multiVideo.autoplay = true;
    multiVideo.id = "multiVideo-" + id;
    videoContainner.appendChild(multiVideo);

    let canvas = document.createElement("canvas");
    canvas.id = "canvas-" + id;
    videoContainner.appendChild(canvas);

    let messageBox = document.createElement("div");
    messageBox.id = "messageBox-" + id;
    messageBox.class = 'messagebox';
    let p = document.createElement("p");
    p.innerHTML = '현재 인터넷 환경이 좋지 않아 영상 수신을 잠시 중단합니다.';
    messageBox.appendChild(p);
    videoContainner.appendChild(messageBox);
    messageBox.hidden = true;

    videoBox.appendChild(videoContainner);
}

//Local stream, peer 생성 및 sdp return
const createSDPOffer = async id => {
    return new Promise(async (resolve, reject) => {
        peers[id] = new RTCPeerConnection();
        streams[id] = await navigator.mediaDevices.getUserMedia({video: true, audio: true});
        let str = 'multiVideo-'+id;
        let multiVideo = document.getElementById(str);
        multiVideo.srcObject = streams[id];
        streams[id].getTracks().forEach(track => {
            peers[id].addTrack(track, streams[id]);
        });

        peers[id].createOffer().then(sdp => {
            peers[id].setLocalDescription(sdp);
            return sdp;
        }).then(sdp => {
            resolve(sdp);
        })
    })
}

//send sdp answer
const createSDPAnswer = async data => {
    let displayId = data.displayId;

    peers[displayId] = new RTCPeerConnection();
    peers[displayId].ontrack = e => {
        streams[displayId] = e.streams[0];

        let multiVideo = document.getElementById(`multiVideo-${displayId}`);
        multiVideo.srcObject = streams[displayId];
    }

    await peers[displayId].setRemoteDescription(data.sdp);
    let answerSdp = await peers[displayId].createAnswer();
    await peers[displayId].setLocalDescription(answerSdp);
    peers[displayId].onicecandidate = e => {
        if(!e.candidate){
            let reqData = {
                "eventOp": "SDP",
                "sdp": peers[displayId].localDescription,
                "roomId": data.roomId,
                "usage": "cam",
                "pluginId": data.pluginId,
                "userId": userId
            };

            sendData(reqData);
        }
    }
}

//퇴장 시, stream,peer 제거
const leaveParticipant = id => {
    document.getElementById(`multiVideo-${id}`).remove();
    document.getElementById(id).remove();

    if(streams[id]){
        streams[id].getVideoTracks()[0].stop();
        streams[id].getAudioTracks()[0].stop();
        streams[id] = null;
        delete streams[id];
    }

    if(peers[id]){
        peers[id].close();
        peers[id] = null;
        delete peers[id];
    }

}

function handleAudioSafeGd() {
    console.log('network speed is good.');

    if (membersOfVideoOff.length > 0) {
        let survivor = membersOfVideoOff.pop();
        membersOfVideoOn.push(survivor);

        sendData({
            eventOp: 'EditFeed',
            roomId: roomIdInput.value,
            userId: survivor,
            video: true,
            audio: true
        })
    }
}

function handleAudioSafeBd() {
    console.log('network speed was not good.');

    if (membersOfVideoOn.length > 1) {
        let victim = membersOfVideoOn.pop();
        membersOfVideoOff.push(victim);

        sendData({
            eventOp: 'EditFeed',
            roomId: roomIdInput.value,
            userId: victim,
            video: false,
            audio: true
        })
    }
}

function handleAudioSafeNbd() {
    console.log('network speed is not bad.');
}

/********************** button event **********************/
CreateRoomBtn.addEventListener('click', () => {
    host = true;
    let data = {
        "eventOp":"CreateRoom"
    }

    sendData(data);
});

RoomJoinBtn.addEventListener('click', () => {
    let data = {
        "eventOp":"RoomJoin",
        "roomId": roomIdInput.value
    }

    sendData(data);
});

SDPBtn.addEventListener('click', async () => {

    let sdp = await createSDPOffer(userId);

    let data = {
        "eventOp":"SDP",
        "pluginId": undefined,
        "roomId": roomIdInput.value,
        "sdp": sdp,
        "usage": "cam",
        "userId": userId
    }

    sendData(data);

    SDPBtn.disabled = true;
    blurBtn.disabled = false;
    speakerDetectBtn.disabled = false;
    audioSafeBtn.disabled = false;
})

audioSafeBtn.addEventListener('click', async () => {
    window.addEventListener('onaudiosafegd', handleAudioSafeGd);
    window.addEventListener('onaudiosafebd', handleAudioSafeBd);
    window.addEventListener('onaudiosafenbd', handleAudioSafeNbd);

    cocolib.audioSafe.start(peers[userId], 3);
    audioSafeBtn.hidden = true;
    audioSafeCancelBtn.hidden = false;
})

audioSafeCancelBtn.addEventListener('click', async () => {
    window.removeEventListener('onaudiosafegd', handleAudioSafeGd);
    window.removeEventListener('onaudiosafebd', handleAudioSafeBd);
    window.removeEventListener('onaudiosafenbd', handleAudioSafeNbd);

    cocolib.audioSafe.stop();
    audioSafeBtn.hidden = false;
    audioSafeCancelBtn.hidden = true;
})

speakerDetectBtn.addEventListener('click', async () => {
    // alert('2초 이상 말하는 사람에게 빨간 테두리가 나옵니다.');
    cocolib.detectSpeaker.start();
    speakerDetectBtn.hidden = true;
    speakerDetectCancelBtn.hidden = false;
})

speakerDetectCancelBtn.addEventListener('click', async () => {
    cocolib.detectSpeaker.stop();
    speakerDetectBtn.hidden = false;
    speakerDetectCancelBtn.hidden = true;
})

blurBtn.addEventListener('click', e => {
    let myVideo = document.getElementById('multiVideo-' + userId);
    let myCanvas = document.getElementById('canvas-' + userId);
    blurBtn.hidden = true;
    unblurBtn.hidden = false;

    myVideo.hidden = true;
    myCanvas.hidden = false;

    cocolib.privacyMode.start(myCanvas, myVideo, {width: 960, height: 540}, 10, 3);

    sendData({
        signalOp: 'SetBlur',
        roomId: roomIdInput.value,
        userId,
        status: true
    });
});

unblurBtn.addEventListener('click', e => {
    let myVideo = document.getElementById('multiVideo-' + userId);
    let myCanvas = document.getElementById('canvas-' + userId);
    blurBtn.hidden = false;
    unblurBtn.hidden = true;

    myVideo.hidden = false;
    myCanvas.hidden = true;

    cocolib.privacyMode.stop();

    sendData({
        signalOp: 'SetBlur',
        roomId: roomIdInput.value,
        userId,
        status: false
    });
});

/********************** event receive **********************/
clientIo.on("knowledgetalk", async data => {

    socketLog('receive', data);

    switch(data.eventOp || data.signalOp) {
        case 'CreateRoom':
            if(data.code == '200'){
                createRoom(data);
                CreateRoomBtn.disabled = true;
            }
            break;

        case 'RoomJoin':
            if(data.code == '200'){
                roomJoin(data);
                RoomJoinBtn.disabled = true;
                CreateRoomBtn.disabled = true;
            }
            break;

        case 'StartSession':
            startSession(data);
            break;

        case 'SDP':
            if(data.useMediaSvr == 'Y'){
                if(data.sdp && data.sdp.type == 'offer'){
                    createSDPAnswer(data);
                }
                else if(data.sdp && data.sdp.type == 'answer'){
                    peers[userId].setRemoteDescription(new RTCSessionDescription(data.sdp));
                }
            }
            break;
        case 'ReceiveFeed':
            receiveFeed(data)
            break;

        case 'Presence':
            if(data.action == 'exit'){
                leaveParticipant(data.userId)
            } else if (data.action === 'talking') {
                // 화자 감지
                let videoBoxDiv = document.getElementById(data.userId);
                if (data.talking && cocolib.detectSpeaker.status) {
                    videoBoxDiv.style.border = '3px solid red';
                } else {
                    videoBoxDiv.style.border = '0px none';
                }
            }

            break;

        case 'SetBlur':
            // 사생활 보호
            let video = document.getElementById('multiVideo-' + data.userId);
            let canvas = document.getElementById('canvas-' + data.userId);
            if (data.status) {
                cocolib.privacyMode.start(canvas, video, {width: 960, height: 540}, 10, 3);
                video.hidden = true;
                canvas.hidden = false;
            } else {
                cocolib.privacyMode.stop();
                video.hidden = false;
                canvas.hidden = true;
            }

            break;

        case 'EditFeed':
            // 오디오 세이프
            let target = document.getElementById('multiVideo-' + data.userId);
            let messageBox = document.getElementById('messageBox-' + data.userId);
            if (data.video) {
                target.hidden = false;
                messageBox.hidden = true;
            } else {
                target.hidden = true;
                messageBox.hidden = false;
            }
            
            break;

    }

});


const createRoom = data => {
    roomIdInput.value = data.roomId;

    //room id copy to clipboard
    roomIdInput.select();
    roomIdInput.setSelectionRange(0, 99999);
    document.execCommand("copy");

    alert('room id copied')
}

const roomJoin = data => {
    userId = data.userId;
}

const startSession = async data => {
    members = Object.keys(data.members);

    //3명 이상일 때, 다자간 통화 연결 시작
    if(data.useMediaSvr == 'Y'){
        for(let i=0; i<members.length; ++i){
            let user = document.getElementById(members[i]);
            if(!user){
                createVideoBox(members[i]);
            }
        }

        SDPBtn.disabled = false;
    }
}

const receiveFeed = (data) => {
    data.feeds.forEach(result => {
        let data = {
            "eventOp":"SendFeed",
            "roomId": roomIdInput.value,
            "usage": "cam",
            "feedId": result.id,
            "display": result.display
        }

        sendData(data);
    })
}
